@@ -55,6 +55,8 @@ class ExportRequest:
     limit: int | None = None
     offset: int = 0
     status_filters: Sequence[str] | None = None
+    review_status_filters: Sequence[str] | None = None
+    include_all_review_statuses: bool = False
     trajectory_event_limit: int = DEFAULT_TRAJECTORY_LIMIT
     batch_size: int = DEFAULT_BATCH_SIZE
     min_connections: int = 1
@@ -72,12 +74,19 @@ async def export_sessions(
     batch_size: int = DEFAULT_BATCH_SIZE,
     trajectory_limit: int | None = DEFAULT_TRAJECTORY_LIMIT,
     status_filters: Sequence[str] | None = None,
+    review_status_filters: Sequence[str] | None = None,
+    include_all_review_statuses: bool = False,
 ) -> ExportSummary:
     """Export sessions from storage into newline-delimited JSON."""
 
     stats = ExportSummary()
     output_path.parent.mkdir(parents=True, exist_ok=True)
     statuses = {status.lower() for status in status_filters or []}
+    review_filters: set[str] | None
+    if include_all_review_statuses:
+        review_filters = None
+    else:
+        review_filters = {status.lower() for status in (review_status_filters or ["approved"])}
     with output_path.open("w", encoding="utf-8") as handle:
         async for payload in _iter_session_payloads(
             store,
@@ -87,6 +96,7 @@ async def export_sessions(
             batch_size=batch_size,
             trajectory_limit=trajectory_limit,
             status_filters=statuses,
+            review_status_filters=review_filters,
         ):
             if payload is None:
                 continue
@@ -106,6 +116,8 @@ async def export_sessions_to_jsonl(
     batch_size: int = DEFAULT_BATCH_SIZE,
     trajectory_limit: int | None = DEFAULT_TRAJECTORY_LIMIT,
     status_filters: Sequence[str] | None = None,
+    review_status_filters: Sequence[str] | None = None,
+    include_all_review_statuses: bool = False,
     min_connections: int = 1,
     max_connections: int = 4,
     statement_timeout_seconds: float = 30.0,
@@ -130,6 +142,8 @@ async def export_sessions_to_jsonl(
             batch_size=batch_size,
             trajectory_limit=trajectory_limit,
             status_filters=status_filters,
+            review_status_filters=review_status_filters,
+            include_all_review_statuses=include_all_review_statuses,
         )
     finally:
         await database.disconnect()
@@ -148,6 +162,8 @@ async def export_sessions_async(request: ExportRequest) -> ExportSummary:
         batch_size=request.batch_size,
         trajectory_limit=request.trajectory_event_limit,
         status_filters=request.status_filters,
+        review_status_filters=request.review_status_filters,
+        include_all_review_statuses=request.include_all_review_statuses,
         min_connections=request.min_connections,
         max_connections=request.max_connections,
         statement_timeout_seconds=request.statement_timeout_seconds,
@@ -158,7 +174,7 @@ def export_sessions_sync(request: ExportRequest) -> ExportSummary:
     """Synchronous helper for CLI usage."""
 
     try:
-        loop = asyncio.get_running_loop()
+        asyncio.get_running_loop()
     except RuntimeError:
         return asyncio.run(export_sessions_async(request))
     raise RuntimeError("export_sessions_sync cannot run within an existing event loop")
@@ -173,6 +189,7 @@ async def _iter_session_payloads(
     batch_size: int,
     trajectory_limit: int | None,
     status_filters: set[str],
+    review_status_filters: set[str] | None,
 ):
     seen: set[int] = set()
     event_limit = (
@@ -194,6 +211,7 @@ async def _iter_session_payloads(
                 session_id,
                 trajectory_limit=event_limit,
                 status_filters=status_filters,
+                review_status_filters=review_status_filters,
             )
             if payload is not None:
                 yield payload
@@ -210,15 +228,20 @@ async def _iter_session_payloads(
         if not rows:
             break
         for row in rows:
-            session_id = row.get("id")
-            if not isinstance(session_id, int):
+            raw_session_id = row.get("id")
+            if not isinstance(raw_session_id, int):
                 continue
+            session_id = raw_session_id
             if session_id in seen:
                 continue
             if status_filters:
                 status_value = str(row.get("status", "")).lower()
                 if status_value not in status_filters:
                     continue
+            if review_status_filters is not None:
+                review_value = str(row.get("review_status", "")).lower()
+                if review_value not in review_status_filters:
+                    continue
             seen.add(session_id)
             payload = await _assemble_session(
                 store,
@@ -226,6 +249,7 @@ async def _iter_session_payloads(
                 preloaded_session=row,
                 trajectory_limit=event_limit,
                 status_filters=status_filters,
+                review_status_filters=review_status_filters,
             )
             if payload is not None:
                 yield payload
@@ -241,6 +265,7 @@ async def _assemble_session(
     preloaded_session: dict[str, Any] | None = None,
     trajectory_limit: int | None = None,
     status_filters: set[str],
+    review_status_filters: set[str] | None,
 ) -> dict[str, Any] | None:
     session_row = preloaded_session or await store.fetch_session(session_id)
     if session_row is None:
@@ -251,6 +276,10 @@ async def _assemble_session(
         status_value = str(session_row.get("status", "")).lower()
         if status_value not in status_filters:
             return None
+    if review_status_filters is not None:
+        review_value = str(session_row.get("review_status", "")).lower()
+        if review_value not in review_status_filters:
+            return None
 
     detailed = session_row
     if "plan" not in detailed or detailed["plan"] is None:
@@ -298,6 +327,34 @@ async def _assemble_session(
         "steps": step_payloads,
         "session_metadata": session_metadata,
     }
+    execution_mode = session_metadata.get("execution_mode")
+    if isinstance(execution_mode, str) and execution_mode:
+        session_payload["execution_mode"] = execution_mode
+    elif isinstance(session_metadata.get("adaptive_summary"), dict):
+        summary_mode = session_metadata["adaptive_summary"].get("adaptive_mode")
+        if isinstance(summary_mode, str) and summary_mode:
+            session_payload["execution_mode"] = summary_mode
+    review_status = detailed.get("review_status") or session_metadata.get("review_status")
+    if isinstance(review_status, str) and review_status:
+        session_payload["review_status"] = review_status
+        session_metadata.setdefault("review_status", review_status)
+    review_notes = detailed.get("review_notes")
+    if isinstance(review_notes, str) and review_notes.strip():
+        session_payload["review_notes"] = review_notes.strip()
+    reward_stats = _coerce_json(detailed.get("reward_stats")) or session_metadata.get("reward_stats")
+    if isinstance(reward_stats, dict) and reward_stats:
+        session_payload["reward_stats"] = reward_stats
+        session_metadata.setdefault("reward_stats", reward_stats)
+    reward_audit_payload = detailed.get("reward_audit")
+    reward_audit = _coerce_json(reward_audit_payload)
+    if not isinstance(reward_audit, list) or not reward_audit:
+        fallback_audit = session_metadata.get("reward_audit")
+        reward_audit = fallback_audit if isinstance(fallback_audit, list) else []
+    if reward_audit:
+        normalized_audit = [entry for entry in reward_audit if isinstance(entry, dict)]
+        if normalized_audit:
+            session_payload["reward_audit"] = normalized_audit
+            session_metadata.setdefault("reward_audit", normalized_audit)
     session_reward = _coerce_json(detailed.get("reward"))
     if isinstance(session_reward, dict) and session_reward:
         session_payload["session_reward"] = session_reward
@@ -313,21 +370,26 @@ async def _assemble_session(
     triage_dossier = session_metadata.get("triage_dossier")
     if isinstance(triage_dossier, dict):
         session_payload["triage_dossier"] = triage_dossier
-    personas_used = session_metadata.get("personas_used")
-    if isinstance(personas_used, list):
-        session_payload["personas_used"] = personas_used
-    persona_updates = session_metadata.get("persona_updates")
-    if persona_updates:
-        session_payload["persona_updates"] = persona_updates
-    persona_usage = session_metadata.get("persona_usage")
-    if isinstance(persona_usage, dict) and persona_usage:
-        session_payload["persona_usage"] = persona_usage
+    learning_key = session_metadata.get("learning_key")
+    if isinstance(learning_key, str) and learning_key:
+        session_payload["learning_key"] = learning_key
+    learning_history = session_metadata.get("learning_history")
+    if isinstance(learning_history, dict) and learning_history:
+        session_payload["learning_history"] = learning_history
     teacher_notes = session_metadata.get("teacher_notes")
     if isinstance(teacher_notes, list):
         session_payload["teacher_notes"] = teacher_notes
     reward_summary = session_metadata.get("reward_summary")
     if isinstance(reward_summary, dict):
         session_payload["reward_summary"] = reward_summary
+    drift_payload = session_metadata.get("drift")
+    if isinstance(drift_payload, dict):
+        session_payload["drift"] = drift_payload
+    drift_alert_value = session_metadata.get("drift_alert")
+    if drift_alert_value is None and isinstance(drift_payload, dict):
+        drift_alert_value = drift_payload.get("drift_alert")
+    if drift_alert_value is not None:
+        session_payload["drift_alert"] = drift_alert_value
     if events_payload:
         session_payload["trajectory_events"] = events_payload
     return session_payload
@@ -387,11 +449,13 @@ def _build_step_payload(
     attempts_value = step_row.get("attempts")
     if isinstance(attempts_value, int):
         attempts = attempts_value
-    else:
+    elif isinstance(attempts_value, (float, str)):
         try:
             attempts = int(attempts_value)
         except (TypeError, ValueError):
             attempts = len(attempt_details) or len(guidance) or 0
+    else:
+        attempts = len(attempt_details) or len(guidance) or 0
 
     metadata = _coerce_metadata(step_row.get("metadata"))
     if attempt_details:
@@ -424,7 +488,7 @@ def _build_step_payload(
         else:
             prior_results_text[key_str] = str(text_value)
 
-    step_payload = {
+    step_payload: dict[str, Any] = {
         "step_id": step_id,
         "description": _extract_plan_field(plan_step, "description") or "",
         "trace": step_row.get("trace") or "",
@@ -448,9 +512,13 @@ def _build_step_payload(
     if runtime_meta and "runtime" not in step_payload:
         step_payload["runtime"] = runtime_meta
 
-    for key, value in metadata.items():
-        if key not in step_payload:
-            step_payload[key] = value
+    extra_fields = {
+        key: value
+        for key, value in metadata.items()
+        if isinstance(key, str) and key not in step_payload
+    }
+    if extra_fields:
+        step_payload.update(extra_fields)
 
     step_payload["metadata"] = metadata
     return step_payload
@@ -568,9 +636,19 @@ def _normalise_event(entry: dict[str, Any]) -> dict[str, Any]:
     event_payload = _coerce_json(payload.get("event")) or {}
     payload["event"] = event_payload
     if isinstance(event_payload, dict):
-        for key in ("type", "name"):
-            if key in event_payload and key not in payload:
-                payload[key] = event_payload[key]
+        event_type = event_payload.get("event_type") or event_payload.get("type")
+        if event_type and "type" not in payload:
+            payload["type"] = event_type
+        if event_type and "event_type" not in payload:
+            payload["event_type"] = event_type
+        name_value = event_payload.get("name")
+        if name_value and "name" not in payload:
+            payload["name"] = name_value
+        metadata_payload = event_payload.get("metadata")
+        if isinstance(metadata_payload, dict):
+            actor = metadata_payload.get("actor")
+            if actor and "actor" not in payload:
+                payload["actor"] = actor
     created_at = payload.get("created_at")
     if created_at is not None:
         payload["created_at"] = _format_timestamp(created_at)
