@@ -4,6 +4,7 @@
 
 from __future__ import annotations
 
+import warnings
 from enum import Enum
 from typing import Any
 from typing import Dict
@@ -18,6 +19,12 @@ from pydantic import Field
 from pydantic import field_validator
 from pydantic import model_validator
 
+warnings.filterwarnings(
+    "ignore",
+    message='Field name "schema" in "LearningConfig" shadows an attribute in parent "BaseModel"',
+    category=UserWarning,
+)
+
 class RetryPolicy(BaseModel):
     """Retry behavior for adapter calls."""
 
@@ -110,6 +117,47 @@ class LLMProvider(str, Enum):
     AZURE_OPENAI = "azure-openai"
     BEDROCK = "bedrock"
     GOOGLE = "google"
+    GEMINI = "gemini"
+    XAI = "xai"
+
+class MetadataDigestConfig(BaseModel):
+    """Controls how execution metadata is projected into LLM-facing prompts.
+
+    Defaults reserve roughly 10%% of the provider's published context window, assuming ~4 characters per token.
+    Override ``char_budget`` or ``provider_char_budgets`` if you need a different policy.
+    """
+
+    model_config = ConfigDict(extra="forbid")
+
+    enabled: bool = True
+    char_budget: int | None = Field(default=None, ge=1024)
+    provider_char_budgets: Dict[LLMProvider, int] = Field(default_factory=dict)
+    include_session_keys: List[str] = Field(
+        default_factory=lambda: [
+            "source",
+            "execution_mode",
+            "adaptive_summary",
+            "token_usage",
+            "reward_summary",
+            "reward_stats",
+            "reward_audit_summary",
+            "triage_dossier",
+            "drift_alert",
+            "learning_usage",
+            "student_learning",
+            "teacher_learning",
+            "session_learning_note",
+            "notes",
+        ]
+    )
+    max_plan_steps: int = Field(default=5, ge=0, le=20)
+    max_step_summaries: int = Field(default=5, ge=0, le=20)
+    max_learning_history_entries: int = Field(default=3, ge=0, le=10)
+    max_reward_audit_entries: int = Field(default=3, ge=0, le=10)
+    max_prompt_rewrite_chars: int = Field(default=2000, ge=256, le=20000)
+    max_section_chars: int = Field(default=4000, ge=512, le=20000)
+    max_string_chars: int = Field(default=1000, ge=128, le=4000)
+
 
 class LLMParameters(BaseModel):
     """Configuration for an LLM request path."""
@@ -135,12 +183,24 @@ class OpenAIAdapterConfig(AdapterConfig):
     type: Literal[AdapterType.OPENAI] = AdapterType.OPENAI
     llm: LLMParameters
     response_format: Dict[str, Any] | None = None
+    metadata_digest: MetadataDigestConfig = Field(default_factory=MetadataDigestConfig)
 
     @field_validator("llm")
     @classmethod
     def ensure_openai_provider(cls, value: LLMParameters):
-        if value.provider not in {LLMProvider.OPENAI, LLMProvider.AZURE_OPENAI}:
-            raise ValueError("openai adapter requires an OpenAI compatible provider")
+        allowed = {
+            LLMProvider.OPENAI,
+            LLMProvider.AZURE_OPENAI,
+            LLMProvider.ANTHROPIC,
+            LLMProvider.GEMINI,
+            LLMProvider.BEDROCK,
+            LLMProvider.XAI,
+        }
+        if value.provider not in allowed:
+            raise ValueError(
+                "openai adapter requires an OpenAI-compatible provider (OpenAI/Azure) or LiteLLM-supported provider "
+                f"({', '.join(provider.value for provider in allowed if provider not in {LLMProvider.OPENAI, LLMProvider.AZURE_OPENAI})})"
+            )
         return value
 
 AdapterUnion = HTTPAdapterConfig | PythonAdapterConfig | OpenAIAdapterConfig
@@ -163,7 +223,7 @@ class TeacherPrompts(BaseModel):
     validation: str
     guidance: str
 
-AdaptiveMode = Literal["auto", "paired", "coach", "escalate"]
+AdaptiveMode = Literal["auto", "paired", "coach"]
 
 
 class AdaptiveProbeThresholds(BaseModel):
@@ -187,9 +247,17 @@ class AdaptiveProbeConfig(BaseModel):
 
     model_config = ConfigDict(extra="forbid")
 
-    llm: "LLMParameters | None" = None
+    llm: "LLMParameters | None" = Field(
+        default_factory=lambda: LLMParameters(
+            provider=LLMProvider.XAI,
+            model="xai/grok-4-fast",
+            api_key_env="XAI_API_KEY",
+            temperature=0.2,
+            timeout_seconds=20.0,
+        )
+    )
     thresholds: AdaptiveProbeThresholds = Field(default_factory=AdaptiveProbeThresholds)
-    fallback_mode: Literal["paired", "coach", "escalate"] = "paired"
+    fallback_mode: Literal["paired", "coach"] = "paired"
     evidence_limit: int = Field(default=6, ge=1, le=32)
     timeout_seconds: float = Field(default=15.0, ge=1.0)
 
@@ -225,6 +293,7 @@ class AdaptiveTeachingConfig(BaseModel):
     default_tags: List[str] = Field(default_factory=list)
     probe: AdaptiveProbeConfig = Field(default_factory=AdaptiveProbeConfig)
     reward: RewardObjectiveConfig = Field(default_factory=RewardObjectiveConfig)
+    learning_history_limit: int = Field(default=10, ge=1, le=200)
 
     @field_validator("default_tags", mode="before")
     @classmethod
@@ -271,6 +340,105 @@ class TeacherConfig(BaseModel):
     prompts: TeacherPrompts | None = None
     prompt_guidance: Dict[str, str] = Field(default_factory=dict)
 
+class LearningPrompts(BaseModel):
+    """Prompt templates for the learning synthesizer."""
+
+    model_config = ConfigDict(extra="forbid")
+
+    synthesizer: str | None = None
+
+
+class PlaybookEntryGateRules(BaseModel):
+    """Gate configuration enforced on generated learning playbook entries."""
+
+    model_config = ConfigDict(extra="forbid")
+
+    enforce_actionability: bool = True
+    enforce_cue: bool = True
+    enforce_generality: bool = False  # Default False - use empirical validation instead
+    max_text_length: int = Field(default=420, ge=100, le=2000)
+    allowed_proper_nouns: List[str] = Field(
+        default_factory=lambda: ["SQL", "HTTP", "JSON", "Atlas", "API"]
+    )
+    banned_incident_tokens: List[str] = Field(
+        default_factory=lambda: ["incident", "ticket", "case", "postmortem"]
+    )
+    allow_length_overflow_margin: int = Field(default=20, ge=0, le=500)
+
+
+class PlaybookEntryRubricWeights(BaseModel):
+    """Weighted scoring rubric applied to learning playbook entries."""
+
+    model_config = ConfigDict(extra="forbid")
+
+    actionability: float = Field(default=0.4, ge=0.0, le=1.0)
+    generality: float = Field(default=0.3, ge=0.0, le=1.0)
+    hookability: float = Field(default=0.2, ge=0.0, le=1.0)
+    concision: float = Field(default=0.1, ge=0.0, le=1.0)
+
+
+class PlaybookPruningConfig(BaseModel):
+    """Configuration for empirical pruning of playbook entries."""
+
+    model_config = ConfigDict(extra="forbid")
+
+    min_sessions: int = Field(default=10, ge=1, le=100)
+    min_cue_hit_rate: float = Field(default=0.05, ge=0.0, le=1.0)
+    min_reward_delta: float = Field(default=0.01, ge=-1.0, le=1.0)
+    min_transfer_sessions: int = Field(default=20, ge=1, le=100)
+
+
+class PlaybookEntrySchemaConfig(BaseModel):
+    """Schema metadata for playbook entry synthesis."""
+
+    model_config = ConfigDict(extra="forbid")
+
+    version: str = "playbook_entry.v1"
+    allowed_runtime_handles: List[str] = Field(default_factory=list)
+    runtime_handle_prefixes: List[str] = Field(default_factory=list)
+    cue_types: List[str] = Field(default_factory=lambda: ["regex", "keyword", "predicate"])
+    default_scope_category: str = "differentiation"
+    allow_missing_tool_mapping: bool = False
+
+
+class LearningUsageConfig(BaseModel):
+    """Runtime usage instrumentation toggles."""
+
+    model_config = ConfigDict(extra="forbid")
+
+    enabled: bool = True
+    capture_examples: bool = False
+    max_examples_per_entry: int = Field(default=2, ge=0, le=20)
+
+
+class LearningConfig(BaseModel):
+    """Controls the learning pamphlet synthesizer."""
+
+    model_config = ConfigDict(extra="forbid")
+
+    enabled: bool = True
+    update_enabled: bool = True
+    provisional_acceptance: bool = Field(default=True, description="Accept entries provisionally even if they fail generality gate")
+    llm: LLMParameters | None = Field(
+        default_factory=lambda: LLMParameters(
+            provider=LLMProvider.GEMINI,
+            model="gemini/gemini-2.5-flash",
+            api_key_env="GEMINI_API_KEY",
+            temperature=0.1,
+            max_output_tokens=8192,
+            timeout_seconds=120.0,
+        )
+    )
+    prompts: LearningPrompts | None = None
+    history_limit: int = Field(default=10, ge=1, le=200)
+    session_note_enabled: bool = True
+    apply_to_prompts: bool = True
+    schema: PlaybookEntrySchemaConfig = Field(default_factory=PlaybookEntrySchemaConfig)
+    rubric_weights: PlaybookEntryRubricWeights = Field(default_factory=PlaybookEntryRubricWeights)
+    gates: PlaybookEntryGateRules = Field(default_factory=PlaybookEntryGateRules)
+    pruning_config: PlaybookPruningConfig = Field(default_factory=PlaybookPruningConfig)
+    usage_tracking: LearningUsageConfig = Field(default_factory=LearningUsageConfig)
+
 class RIMConfig(BaseModel):
     """Aggregate reward model configuration."""
 
@@ -295,6 +463,7 @@ class OrchestrationConfig(BaseModel):
     step_timeout_seconds: float = Field(default=900.0, ge=0.0)
     rim_guidance_tag: str = "rim_feedback"
     emit_intermediate_steps: bool = True
+    forced_mode: AdaptiveMode | None = None
 
 class StorageConfig(BaseModel):
     """PostgreSQL connection settings."""
@@ -305,6 +474,36 @@ class StorageConfig(BaseModel):
     min_connections: int = Field(default=1, ge=1)
     max_connections: int = Field(default=5, ge=1)
     statement_timeout_seconds: float = Field(default=30.0, ge=0.0)
+    apply_schema_on_connect: bool = True
+
+
+class DriftDetectionConfig(BaseModel):
+    """Rolling statistics used to detect reward drift."""
+
+    model_config = ConfigDict(extra="forbid")
+
+    enabled: bool = True
+    window: int = Field(default=50, ge=1)
+    z_threshold: float = Field(default=3.0, ge=0.0)
+    min_baseline: int = Field(default=5, ge=0)
+
+
+class ReviewWorkflowConfig(BaseModel):
+    """Review gating applied before exporting traces."""
+
+    model_config = ConfigDict(extra="forbid")
+
+    require_approval: bool = True
+    default_export_statuses: List[str] = Field(default_factory=lambda: ["approved"])
+
+
+class RuntimeSafetyConfig(BaseModel):
+    """Top-level guardrail configuration."""
+
+    model_config = ConfigDict(extra="forbid")
+
+    drift: DriftDetectionConfig = Field(default_factory=DriftDetectionConfig)
+    review: ReviewWorkflowConfig = Field(default_factory=ReviewWorkflowConfig)
 
 class AtlasConfig(BaseModel):
     """Root configuration consumed by the Atlas SDK."""
@@ -316,7 +515,9 @@ class AtlasConfig(BaseModel):
     teacher: TeacherConfig
     orchestration: OrchestrationConfig = Field(default_factory=OrchestrationConfig)
     rim: RIMConfig
+    learning: LearningConfig = Field(default_factory=LearningConfig)
     storage: StorageConfig | None = None
     prompt_rewrite: PromptRewriteConfig | None = None
     metadata: Dict[str, Any] = Field(default_factory=dict)
     adaptive_teaching: AdaptiveTeachingConfig = Field(default_factory=AdaptiveTeachingConfig)
+    runtime_safety: RuntimeSafetyConfig = Field(default_factory=RuntimeSafetyConfig)
