@@ -24,14 +24,15 @@ from atlas.runtime.models.intermediate_step import StreamEventData
 from atlas.runtime.orchestration.execution_context import ExecutionContext
 from atlas.config.models import AdapterConfig
 from atlas.config.models import StudentConfig
-from atlas.config.models import ToolDefinition
-from atlas.connectors.langchain_bridge import BYOABridgeLLM
 from atlas.connectors.langchain_bridge import build_bridge
+from atlas.connectors.utils import normalise_usage_payload
 from atlas.runtime.agent_loop.tool_loop import ToolCallAgentGraph
 from atlas.runtime.agent_loop.tool_loop import ToolCallAgentGraphState
 from atlas.prompts import RewrittenStudentPrompts
 from atlas.types import Plan
 from atlas.types import Step
+from atlas.learning.playbook import resolve_playbook
+from atlas.learning.usage import get_tracker
 
 logger = logging.getLogger(__name__)
 
@@ -61,23 +62,95 @@ class Student:
         adapter_config: AdapterConfig,
         student_config: StudentConfig,
         student_prompts: RewrittenStudentPrompts,
+        *,
+        apply_learning_prompts: bool = True,
     ) -> None:
         self._adapter = adapter
         self._student_config = student_config
         self._prompts: RewrittenStudentPrompts = student_prompts
-        self._bridge_llm, self._tools = build_bridge(adapter, adapter_config.tools)
+        self._apply_learning_prompts = apply_learning_prompts
+        self._bridge_llm, self._tools = build_bridge(adapter, adapter_config.tools, tool_choice=self._student_config.tool_choice)
         self._graph: Any | None = None
-        self._graph_builder = ToolCallAgentGraph(
-            llm=self._bridge_llm,
-            tools=self._tools,
-            system_prompt=self._prompts.executor,
-            callbacks=None,
-            detailed_logs=False,
-            log_response_max_chars=1000,
-            handle_tool_errors=True,
-            return_direct=None,
-        )
+        self._graph_builder: ToolCallAgentGraph | None = None
+        self._graph_system_prompt: str | None = None
+        self._refresh_graph_builder()
         self._llm_stream_state: Dict[str, Dict[str, Any]] = {}
+        self._record_runtime_handles()
+
+    def _record_runtime_handles(self, additional_handles: List[str] | None = None) -> None:
+        """Record available tool handles for downstream instrumentation.
+
+        This method uses a three-tier approach:
+        1. Configured tools (self._tools) - for raw adapters with explicit tool config
+        2. Runtime tool calls (additional_handles) - extracted from AIMessage.tool_calls
+        3. Config fallback (allowed_runtime_handles) - for agentic adapters like MCP/LangGraph
+
+        This hybrid approach ensures both raw OpenAI/Anthropic adapters and agentic
+        Python adapters (LangGraph, MCP) can populate runtime handles correctly.
+
+        Args:
+            additional_handles: Optional list of handles from actual tool calls
+        """
+        try:
+            context = ExecutionContext.get()
+        except Exception:  # pragma: no cover - context may be missing in tests
+            return
+        handles: List[str] = []
+        # Tier 1: Add configured tools (for raw adapters)
+        for tool in self._tools or []:
+            name = getattr(tool, "name", None)
+            if isinstance(name, str) and name.strip():
+                handles.append(name.strip())
+        # Tier 2: Add additional handles (for agentic adapters from tool calls)
+        if additional_handles:
+            handles.extend(str(h) for h in additional_handles if isinstance(h, str) and h.strip())
+        # Tier 3: Fallback to config (for agentic adapters that don't expose tool_calls in messages)
+        if not handles:
+            allowed = context.metadata.get("allowed_runtime_handles", [])
+            if isinstance(allowed, list):
+                handles.extend(str(h) for h in allowed if isinstance(h, str) and h.strip())
+        if not handles:
+            return
+        store = context.metadata.setdefault("runtime_handles", [])
+        if not isinstance(store, list):
+            context.metadata["runtime_handles"] = list(handles)
+            return
+        for handle in handles:
+            if handle not in store:
+                store.append(handle)
+
+    def _apply_usage_payload(self, usage: Any) -> Dict[str, int] | None:
+        normalised = normalise_usage_payload(usage)
+        if not isinstance(normalised, dict):
+            return None
+        def _coerce(value: Any) -> int | None:
+            if isinstance(value, (int, float)):
+                return int(value)
+            return None
+        prompt_tokens = _coerce(normalised.get("prompt_tokens"))
+        completion_tokens = _coerce(normalised.get("completion_tokens"))
+        total_tokens = _coerce(normalised.get("total_tokens"))
+        if total_tokens is None and (prompt_tokens is not None or completion_tokens is not None):
+            total_tokens = (prompt_tokens or 0) + (completion_tokens or 0)
+        if prompt_tokens is None and completion_tokens is None and total_tokens is None:
+            return None
+        context = ExecutionContext.get()
+        totals = context.metadata.setdefault(
+            "token_usage",
+            {"prompt_tokens": 0, "completion_tokens": 0, "total_tokens": 0, "calls": 0},
+        )
+        if prompt_tokens is not None:
+            totals["prompt_tokens"] = int(totals.get("prompt_tokens", 0)) + prompt_tokens
+        if completion_tokens is not None:
+            totals["completion_tokens"] = int(totals.get("completion_tokens", 0)) + completion_tokens
+        if total_tokens is not None:
+            totals["total_tokens"] = int(totals.get("total_tokens", 0)) + total_tokens
+        totals["calls"] = int(totals.get("calls", 0)) + 1
+        return {
+            "prompt_tokens": prompt_tokens,
+            "completion_tokens": completion_tokens,
+            "total_tokens": total_tokens,
+        }
 
     def update_prompts(self, student_prompts: RewrittenStudentPrompts) -> None:
         """Refresh student prompts and rebuild execution graph to honour new instructions."""
@@ -89,16 +162,7 @@ class Student:
             return
         self._prompts = student_prompts
         self._graph = None
-        self._graph_builder = ToolCallAgentGraph(
-            llm=self._bridge_llm,
-            tools=self._tools,
-            system_prompt=self._prompts.executor,
-            callbacks=None,
-            detailed_logs=False,
-            log_response_max_chars=1000,
-            handle_tool_errors=True,
-            return_direct=None,
-        )
+        self._refresh_graph_builder()
 
     async def acreate_plan(self, task: str) -> Plan:
         context = ExecutionContext.get()
@@ -117,6 +181,8 @@ class Student:
         prompt = self._compose_planner_prompt(task)
         try:
             response = await self._adapter.ainvoke(prompt, metadata={"mode": "planning"})
+            self._apply_usage_payload(getattr(response, "usage", None))
+            response = self._unwrap_adapter_payload(response)
             if isinstance(response, (dict, list)):
                 payload = response
             else:
@@ -223,6 +289,12 @@ class Student:
                         },
                     }
                 )
+        additional_kwargs = getattr(output_message, "additional_kwargs", None)
+        usage_metadata = normalise_usage_payload(additional_kwargs.get("usage")) if isinstance(additional_kwargs, dict) else None
+        if usage_metadata:
+            if "usage" not in metadata:
+                self._apply_usage_payload(usage_metadata)
+            metadata["usage"] = usage_metadata
         structured_output = self._normalise_executor_message(output_message)
         artifacts = structured_output.get("artifacts") or {}
         if not isinstance(artifacts, dict):
@@ -241,6 +313,7 @@ class Student:
             metadata["reason"] = guidance_reason
         if raw_text is not None:
             metadata["text"] = raw_text
+        self._record_student_action_adoption(step, final_state.messages, structured_output, metadata)
         output_payload = raw_text if isinstance(raw_text, str) else json.dumps(structured_output, ensure_ascii=False)
         return StudentStepResult(
             trace=trace,
@@ -267,6 +340,8 @@ class Student:
         prompt = self._compose_synthesis_prompt(task, step_results)
         try:
             response = await self._adapter.ainvoke(prompt, metadata={"mode": "synthesis"})
+            self._apply_usage_payload(getattr(response, "usage", None))
+            response = self._unwrap_adapter_payload(response)
             if isinstance(response, str):
                 final_answer = response
             else:
@@ -314,25 +389,28 @@ class Student:
             "    {\n"
             "      \"id\": integer,\n"
             "      \"description\": string,\n"
-            "      \"tool\": string | null,\n"
-            "      \"tool_params\": object | null,\n"
             "      \"depends_on\": [integer]\n"
             "    }\n"
             "  ]\n"
             "}\n"
-            "Do not include any prose before or after the JSON object."
+            "Do not include any prose before or after the JSON object.\n"
         )
-        return "\n\n".join([
-            self._prompts.planner,
+        planner_prompt = self._compose_system_prompt(self._prompts.planner, "Student Playbook")
+        components = [
+            planner_prompt,
             f"Task: {task.strip()}",
             json_direction,
-        ])
+        ]
+        return "\n\n".join(components)
 
     def _compose_synthesis_prompt(self, task: str, step_results: List[Dict[str, Any]]) -> str:
         serialized_results = json.dumps(step_results, ensure_ascii=False, indent=2)
         execution_mode = ExecutionContext.get().metadata.get("execution_mode", "stepwise")
+        synthesizer_prompt = self._compose_system_prompt(
+            self._prompts.synthesizer, "Student Playbook"
+        )
         return "\n\n".join([
-            self._prompts.synthesizer,
+            synthesizer_prompt,
             f"Original Task: {task.strip()}",
             f"Execution Mode: {execution_mode}",
             f"Completed Steps: {serialized_results}",
@@ -362,17 +440,140 @@ class Student:
             payload.append("Original Task:")
             payload.append(task_text)
         payload.extend([
-            f"Tool: {step.tool or 'none'}",
-            f"Tool Parameters: {json.dumps(step.tool_params or {}, ensure_ascii=False)}",
             f"Dependencies: {step.depends_on}",
             f"Validated Prior Results (artifacts when available): {context_block}",
             f"Guidance History: {guidance_block}",
         ])
         user_message = "\n".join(payload)
-        return [
-            SystemMessage(content=self._prompts.executor),
+        executor_prompt = self._compose_system_prompt(self._prompts.executor, "Student Playbook")
+        self._refresh_graph_builder()
+        messages = [
+            SystemMessage(content=executor_prompt),
             HumanMessage(content=user_message),
         ]
+        self._record_student_cue_hits(user_message, step.id)
+        return messages
+
+    def _refresh_graph_builder(self) -> None:
+        prompt = self._compose_system_prompt(self._prompts.executor, "Student Playbook")
+        if self._graph_builder is not None and self._graph_system_prompt == prompt:
+            return
+        self._graph_system_prompt = prompt
+        self._graph_builder = ToolCallAgentGraph(
+            llm=self._bridge_llm,
+            tools=self._tools,
+            system_prompt=prompt or None,
+            callbacks=None,
+            detailed_logs=False,
+            log_response_max_chars=1000,
+            handle_tool_errors=True,
+            return_direct=None,
+        )
+        self._graph = None
+
+    def _compose_system_prompt(self, base_prompt: str, label: str) -> str:
+        playbook, _, metadata = resolve_playbook("student", apply=self._apply_learning_prompts)
+        block = self._format_playbook_block(label, playbook, metadata, role="student")
+        base = base_prompt.strip()
+        segments = [segment for segment in (block, base) if segment]
+        return "\n\n".join(segments) if segments else ""
+
+    def _record_student_cue_hits(self, text: str, step_id: int) -> None:
+        if not text:
+            return
+        try:
+            tracker = get_tracker()
+            tracker.detect_and_record("student", text, step_id=step_id, context_hint=text)
+        except Exception:  # pragma: no cover - instrumentation best effort
+            logger.debug("Failed to record cue hits for step %s", step_id, exc_info=True)
+
+    def _record_student_action_adoption(
+        self,
+        step: Step,
+        messages: Sequence[BaseMessage],
+        structured_output: Dict[str, Any],
+        metadata: Dict[str, Any],
+    ) -> None:
+        try:
+            tracker = get_tracker()
+        except Exception:  # pragma: no cover - instrumentation must not break execution
+            logger.debug("Unable to initialise learning usage tracker for step %s", step.id, exc_info=True)
+            return
+        if not getattr(tracker, "enabled", False):
+            return
+        runtime_handles: set[str] = set()
+        for message in messages or []:
+            if isinstance(message, AIMessage) and getattr(message, "tool_calls", None):
+                for call in message.tool_calls:
+                    # Handle both dict (TypedDict from LangChain) and object forms
+                    if isinstance(call, dict):
+                        name = call.get("name")
+                    else:
+                        name = getattr(call, "name", None)
+                    if isinstance(name, str) and name.strip():
+                        runtime_handles.add(name.strip())
+        # Populate runtime_handles for learning system (from actual tool calls)
+        # This supplements the handles recorded during init (from self._tools)
+        # Agentic adapters don't have tools configured, so we extract from actual tool calls
+        if runtime_handles:
+            try:
+                self._record_runtime_handles(additional_handles=list(runtime_handles))
+            except Exception:
+                logger.debug("Failed to record runtime handles for step %s", step.id, exc_info=True)
+        if not runtime_handles:
+            return
+        status_value = (structured_output.get("status") or metadata.get("status") or "").lower()
+        success = status_value in {"ok", "success", "completed", "done"}
+        if not success:
+            success = bool(structured_output.get("deliverable"))
+        adoption_meta = {
+            "status": structured_output.get("status"),
+            "step_id": step.id,
+        }
+        for handle in runtime_handles:
+            tracker.record_action_adoption(
+                "student",
+                handle,
+                success=success,
+                step_id=step.id,
+                metadata=adoption_meta,
+            )
+
+    def _format_playbook_block(
+        self,
+        label: str,
+        playbook: str | None,
+        metadata: Dict[str, Any] | None,
+        *,
+        role: str,
+    ) -> str:
+        if not playbook:
+            return ""
+        header_lines = [f">>> {label} >>>"]
+        metadata_line = self._playbook_metadata_line(metadata, role)
+        if metadata_line:
+            header_lines.append(metadata_line)
+        header_lines.append(playbook)
+        header_lines.append(f">>> End {label} >>>")
+        return "\n".join(header_lines).strip()
+
+    def _playbook_metadata_line(self, metadata: Dict[str, Any] | None, role: str) -> str:
+        if not isinstance(metadata, dict):
+            return ""
+        entries: List[str] = []
+        timestamp_keys = [f"{role}_updated_at", "updated_at", "timestamp", "last_updated"]
+        version_keys = [f"{role}_version", "version"]
+        for key in timestamp_keys:
+            value = metadata.get(key)
+            if isinstance(value, str) and value.strip():
+                entries.append(f"Last updated: {value.strip()}")
+                break
+        for key in version_keys:
+            value = metadata.get(key)
+            if isinstance(value, str) and value.strip():
+                entries.append(f"Version: {value.strip()}")
+                break
+        return " | ".join(entries)
 
     def _build_trace(self, messages: Sequence[BaseMessage]) -> str:
         parts = []
@@ -746,6 +947,15 @@ class Student:
                                 },
                             }
                         )
+                usage_payload = None
+                if isinstance(serialized_output, dict):
+                    extra = serialized_output.get("additional_kwargs")
+                    if isinstance(extra, dict):
+                        usage_payload = extra.get("usage")
+                usage_snapshot = self._apply_usage_payload(usage_payload) if usage_payload is not None else None
+                if usage_snapshot:
+                    metadata = dict(metadata)
+                    metadata["usage"] = usage_snapshot
                 manager.push_intermediate_step(
                     IntermediateStepPayload(
                         UUID=run_id,
@@ -841,8 +1051,9 @@ class Student:
         payload: Dict[str, Any] = {
             "type": message.type,
             "content": message.content,
-            "additional_kwargs": getattr(message, "additional_kwargs", None),
         }
+        additional = getattr(message, "additional_kwargs", None)
+        payload["additional_kwargs"] = self._jsonify(additional)
         tool_calls = getattr(message, "tool_calls", None)
         if tool_calls:
             payload["tool_calls"] = [
@@ -862,6 +1073,44 @@ class Student:
             payload["status"] = status
         return payload
 
+    def _unwrap_adapter_payload(self, payload: Any) -> Any:
+        """Extract textual content from adapter responses carrying auxiliary metadata."""
+        if isinstance(payload, str) and hasattr(payload, "tool_calls"):
+            tool_calls = getattr(payload, "tool_calls", None)
+            if tool_calls:
+                first_call = tool_calls[0] if isinstance(tool_calls, list) and tool_calls else None
+                if isinstance(first_call, dict):
+                    arguments = first_call.get("arguments")
+                    if isinstance(arguments, str) and arguments.strip():
+                        return arguments
+                    if isinstance(arguments, (dict, list)):
+                        return arguments
+                return json.dumps(tool_calls)
+            return str(payload)
+        if not isinstance(payload, dict):
+            return payload
+        content = payload.get("content")
+        tool_calls = payload.get("tool_calls")
+        if tool_calls:
+            if isinstance(content, str) and content.strip():
+                return content
+            first_call = tool_calls[0] if isinstance(tool_calls, list) and tool_calls else None
+            if isinstance(first_call, dict):
+                arguments = first_call.get("arguments")
+                if isinstance(arguments, str) and arguments.strip():
+                    return arguments
+                if isinstance(arguments, (dict, list)):
+                    return arguments
+            return json.dumps(tool_calls)
+        allowed_keys = {"content", "usage"}
+        if (
+            "content" in payload
+            and isinstance(content, (str, list, dict))
+            and set(payload.keys()).issubset(allowed_keys)
+        ):
+            return content
+        return payload
+
     def _extract_reasoning_metadata(self, messages: Sequence[BaseMessage]) -> Dict[str, Any]:
         reasoning_entries: List[Dict[str, Any]] = []
         for index, message in enumerate(messages):
@@ -882,7 +1131,8 @@ class Student:
         return {"reasoning": reasoning_entries}
 
     async def _ensure_graph(self):
-        if self._graph is None:
+        self._refresh_graph_builder()
+        if self._graph is None and self._graph_builder is not None:
             self._graph = await self._graph_builder.build_graph()
         return self._graph
 
