@@ -8,12 +8,10 @@ import logging
 import time
 from dataclasses import dataclass
 from typing import Any
-from typing import Awaitable
 from typing import Callable
 from typing import Dict
 from typing import List
 from typing import Optional
-from typing import Set
 from uuid import uuid4
 
 from atlas.config.models import AdaptiveTeachingConfig
@@ -56,7 +54,6 @@ class _StepExecutionOutcome:
 class AdaptiveModeDecision:
     mode: str
     confidence: Optional[float] = None
-    certification: bool = False
     probe: CapabilityProbeDecision | None = None
     source: str | None = None
 
@@ -71,7 +68,6 @@ class Orchestrator:
         rim_config: RIMConfig,
         adaptive_config: AdaptiveTeachingConfig | None = None,
         triage_adapter: Callable[[str, Dict[str, Any] | None], TriageDossier] | None = None,
-        persona_refresh: Callable[[], Awaitable[None]] | None = None,
         capability_probe: CapabilityProbeClient | None = None,
     ) -> None:
         self._teacher = teacher
@@ -80,14 +76,12 @@ class Orchestrator:
         self._orchestration = orchestration_config
         self._rim_config = rim_config
         self._rim_retry_threshold = getattr(rim_config, "retry_threshold", 0.6)
-        self._persona_refresh = persona_refresh
         self._adaptive = adaptive_config or AdaptiveTeachingConfig()
         self._triage_adapter = triage_adapter or default_build_dossier
         self._capability_probe = capability_probe
 
     async def arun(self, task: str) -> Result:
         context = ExecutionContext.get()
-        context.mark_certification_run(False)
         manager = context.intermediate_step_manager
         orchestration_id = str(uuid4())
         manager.push_intermediate_step(
@@ -133,12 +127,12 @@ class Orchestrator:
         )
         context.metadata["task"] = task
 
-        if self._persona_refresh is not None:
-            await self._persona_refresh()
-        self._hydrate_certified_fingerprints(context)
-
-        decision = await self._determine_adaptive_mode(task, context, dossier)
-        mode = decision.mode or "escalate"
+        forced_mode = getattr(self._orchestration, "forced_mode", None)
+        if forced_mode in {"auto", "paired"}:
+            decision = AdaptiveModeDecision(mode=forced_mode, confidence=1.0, source="forced")
+        else:
+            decision = await self._determine_adaptive_mode(task, context, dossier)
+        mode = decision.mode or "paired"
         context.metadata["execution_mode"] = mode
         self._store_mode_metadata(context, decision)
         confidence_display = f"{decision.confidence:.2f}" if isinstance(decision.confidence, (int, float)) else "n/a"
@@ -260,23 +254,38 @@ class Orchestrator:
             prior_guidance = list(step_meta.get("guidance", []))
 
             structured_output = self._obtain_structured_output(student_result)
-            student_status = structured_output.get("status")
             artifacts = student_result.artifacts
             deliverable = student_result.deliverable
 
             is_final_attempt = allow_retry and attempts > self._orchestration.max_retries
 
             if require_validation and not is_final_attempt:
-                validation_start = time.perf_counter()
-                validation = await self._teacher.avalidate_step(
+                cache_key = self._teacher.validation_signature(
                     step,
-                    student_result.trace,
                     structured_output,
                     context_outputs,
                     prior_guidance,
                     guidance,
                 )
-                attempt_timings["validation_ms"] = self._elapsed_ms(validation_start)
+                validation_cache = execution_context.metadata.setdefault("validation_cache", {})
+                cached_validation = validation_cache.get(cache_key)
+                if cached_validation is not None:
+                    validation = dict(cached_validation)
+                    validation["cached"] = True
+                    attempt_timings["validation_ms"] = 0.0
+                else:
+                    validation_start = time.perf_counter()
+                    validation = await self._teacher.avalidate_step(
+                        step,
+                        student_result.trace,
+                        structured_output,
+                        context_outputs,
+                        prior_guidance,
+                        guidance,
+                    )
+                    attempt_timings["validation_ms"] = self._elapsed_ms(validation_start)
+                    validation.setdefault("cached", False)
+                    validation_cache[cache_key] = self._prepare_cached_validation(validation)
                 logger.info(
                     "Orchestrator: teacher validation for step %s attempt %d=%s (guidance=%s)",
                     step.id,
@@ -285,17 +294,13 @@ class Orchestrator:
                     validation.get("guidance") if isinstance(validation, dict) else None,
                 )
                 validation_valid = bool(validation.get("valid"))
-                reward_override_payload = validation.pop("certification_reward", None) if isinstance(validation, dict) else None
             else:
                 validation = {"valid": True, "guidance": None}
-                reward_override_payload = None
                 attempt_timings["validation_ms"] = 0.0
                 validation_valid = True
 
             reward_skipped = True
-            if reward_override_payload is not None:
-                reward = self._build_placeholder_reward("teacher_certified")
-            elif require_validation and not validation_valid:
+            if require_validation and not validation_valid:
                 reward = self._build_placeholder_reward("validation_failed")
             else:
                 reward = self._build_placeholder_reward("deferred")
@@ -325,11 +330,7 @@ class Orchestrator:
             status_label = "completed"
             if require_validation:
                 status_label = "passed" if validation_valid else "failed"
-            combined_status = (
-                student_status.strip()
-                if isinstance(student_status, str) and student_status.strip()
-                else status_label
-            )
+            combined_status = status_label
 
             execution_context.register_step_attempt(
                 step.id,
@@ -341,9 +342,7 @@ class Orchestrator:
             )
 
             context_entry = self._build_context_entry(structured_output, student_result.output)
-            context_entry["status"] = combined_status
             step_meta["context_entry"] = context_entry
-            step_meta["status"] = combined_status
 
             event_output = {
                 "trace": student_result.trace,
@@ -390,7 +389,7 @@ class Orchestrator:
     ) -> AdaptiveModeDecision:
         adaptive_cfg = self._adaptive
         if not adaptive_cfg.enabled:
-            return AdaptiveModeDecision(mode="escalate", source="adaptive_disabled")
+            return AdaptiveModeDecision(mode="paired", source="adaptive_disabled")
 
         if adaptive_cfg.mode_override:
             return AdaptiveModeDecision(
@@ -399,56 +398,46 @@ class Orchestrator:
                 source="mode_override",
             )
 
-        fingerprint = context.metadata.get("persona_fingerprint")
-        certification_needed = (
-            adaptive_cfg.certify_first_run
-            and isinstance(fingerprint, str)
-            and not self._has_certified_fingerprint(fingerprint)
-        )
-        if certification_needed:
-            decision = AdaptiveModeDecision(
+        learning_history = context.metadata.get("learning_history") if isinstance(context.metadata, dict) else None
+        history_count: int | None = None
+        if isinstance(learning_history, dict):
+            raw_count = learning_history.get("count")
+            if isinstance(raw_count, (int, float)):
+                history_count = int(raw_count)
+        if history_count is None or history_count == 0:
+            logger.info("Orchestrator: no learning history detected; defaulting to paired mode without probe")
+            return AdaptiveModeDecision(
                 mode="paired",
                 confidence=None,
-                certification=True,
-                source="certification_required",
+                source="no_learning_history",
             )
-            return decision
 
-        probe_result = await self._run_capability_probe(task, context, dossier, fingerprint)
+        probe_result = await self._run_capability_probe(task, context, dossier)
         if probe_result is not None:
             mode = probe_result.mode
             confidence = probe_result.confidence
-            if mode not in {"auto", "paired", "coach", "escalate"}:
+            if mode not in {"auto", "paired", "coach"}:
                 mode = self._map_confidence_to_mode(confidence)
             if not mode:
                 fallback_mode = adaptive_cfg.probe.fallback_mode
-                mode = fallback_mode if fallback_mode in {"paired", "coach", "escalate"} else "paired"
-            decision = AdaptiveModeDecision(
+                mode = fallback_mode if fallback_mode in {"paired", "coach"} else "paired"
+            return AdaptiveModeDecision(
                 mode=mode,
                 confidence=confidence,
-                certification=False,
                 probe=probe_result,
                 source="capability_probe",
             )
-            if isinstance(fingerprint, str):
-                self._record_certification_state(fingerprint, False)
-            return decision
 
-        fallback_mode = self._capability_probe.fallback_mode if self._capability_probe else adaptive_cfg.probe.fallback_mode
-        if isinstance(fingerprint, str):
-            self._record_certification_state(fingerprint, False)
+        fallback_mode = adaptive_cfg.probe.fallback_mode
         fallback_source = "probe_disabled" if self._capability_probe is None else "probe_fallback"
-        return AdaptiveModeDecision(
-            mode=fallback_mode if fallback_mode in {"paired", "coach", "escalate"} else "paired",
-            source=fallback_source,
-        )
+        mode = fallback_mode if fallback_mode in {"paired", "coach"} else "paired"
+        return AdaptiveModeDecision(mode=mode, source=fallback_source)
 
     async def _run_capability_probe(
         self,
         task: str,
         context: ExecutionContext,
         dossier: TriageDossier,
-        fingerprint: Any,
     ) -> CapabilityProbeDecision | None:
         manager = context.intermediate_step_manager
         if self._capability_probe is None:
@@ -486,7 +475,6 @@ class Orchestrator:
             decision = await self._capability_probe.arun(
                 task=task,
                 dossier=dossier.model_dump(),
-                fingerprint=str(fingerprint) if isinstance(fingerprint, str) else None,
                 execution_metadata=context.metadata,
             )
         except Exception as exc:
@@ -536,19 +524,13 @@ class Orchestrator:
             return "paired"
         if confidence >= thresholds.coach:
             return "coach"
-        return "escalate"
+        return "coach"
 
     def _store_mode_metadata(self, context: ExecutionContext, decision: AdaptiveModeDecision) -> None:
         context.record_mode_decision(
             decision.mode,
             confidence=decision.confidence,
-            certification=decision.certification,
         )
-        if decision.certification:
-            context.mark_certification_run(True)
-            fingerprint = context.metadata.get("persona_fingerprint")
-            if isinstance(fingerprint, str):
-                self._record_certification_state(fingerprint, True)
 
     def _emit_adaptive_route_event(
         self,
@@ -570,7 +552,6 @@ class Orchestrator:
         payload: Dict[str, Any] = {
             "mode": decision.mode,
             "confidence": decision.confidence,
-            "certification": decision.certification,
         }
         if reason:
             payload["reason"] = reason
@@ -613,47 +594,6 @@ class Orchestrator:
                 pass
         return plan
 
-    def _hydrate_certified_fingerprints(self, context: ExecutionContext) -> None:
-        existing = context.metadata.get("adaptive_certified_fingerprints")
-        if isinstance(existing, set):
-            return
-        certified: Set[str] = set()
-        persona_memories = context.metadata.get("persona_memories") if isinstance(context.metadata, dict) else None
-        if isinstance(persona_memories, dict):
-            for records in persona_memories.values():
-                if not isinstance(records, list):
-                    continue
-                for record in records:
-                    fingerprint = record.get("trigger_fingerprint")
-                    metadata = record.get("metadata")
-                    if isinstance(fingerprint, str) and isinstance(metadata, dict) and metadata.get("certified"):
-                        certified.add(fingerprint)
-        context.metadata["adaptive_certified_fingerprints"] = certified
-
-    def _has_certified_fingerprint(self, fingerprint: str) -> bool:
-        context = ExecutionContext.get()
-        certs = context.metadata.get("adaptive_certified_fingerprints")
-        if isinstance(certs, set):
-            return fingerprint in certs
-        if isinstance(certs, list):
-            return fingerprint in certs
-        return False
-
-    def _record_certification_state(self, fingerprint: str, certified: bool) -> None:
-        context = ExecutionContext.get()
-        certs = context.metadata.setdefault("adaptive_certified_fingerprints", set())
-        if isinstance(certs, set):
-            if certified:
-                certs.add(fingerprint)
-            return
-        if isinstance(certs, list):
-            if certified and fingerprint not in certs:
-                certs.append(fingerprint)
-            return
-        # migrate legacy/non-iterable cases
-        if certified:
-            context.metadata["adaptive_certified_fingerprints"] = {fingerprint}
-
     async def _run_single_shot(
         self,
         task: str,
@@ -766,62 +706,50 @@ class Orchestrator:
                 _store_outcome(step, outcome)
             else:
                 steps = [self._lookup_step(plan, step_id) for step_id in level]
-                if execution_mode == "escalate":
-                    for step in steps:
-                        outcome = await self._run_step(
-                            task,
-                            step,
-                            context_outputs,
-                            context,
-                            require_validation=True,
-                            allow_retry=True,
+                tasks = [
+                    self._run_step(
+                        task,
+                        step,
+                        dict(context_outputs),
+                        context,
+                        require_validation=True,
+                        allow_retry=True,
+                    )
+                    for step in steps
+                ]
+                results = await asyncio.gather(*tasks, return_exceptions=True)
+                captured_exception: Exception | None = None
+                for step, outcome in zip(steps, results):
+                    if isinstance(outcome, Exception):
+                        evaluation = self._build_error_evaluation(str(outcome))
+                        step_summaries.append(
+                            {
+                                "step_id": step.id,
+                                "description": step.description,
+                                "output": "",
+                                "trace": "",
+                                "evaluation": evaluation.to_dict(),
+                                "metadata": {},
+                                "attempts": 0,
+                            }
                         )
-                        _store_outcome(step, outcome)
-                else:
-                    tasks = [
-                        self._run_step(
-                            task,
-                            step,
-                            dict(context_outputs),
-                            context,
-                            require_validation=True,
-                            allow_retry=True,
+                        step_results.append(
+                            StepResult(
+                                step_id=step.id,
+                                trace="",
+                                output="",
+                                evaluation=evaluation,
+                                attempts=0,
+                                metadata={},
+                            )
                         )
-                        for step in steps
-                    ]
-                    results = await asyncio.gather(*tasks, return_exceptions=True)
-                    captured_exception: Exception | None = None
-                    for step, outcome in zip(steps, results):
-                        if isinstance(outcome, Exception):
-                            evaluation = self._build_error_evaluation(str(outcome))
-                            step_summaries.append(
-                                {
-                                    "step_id": step.id,
-                                    "description": step.description,
-                                    "output": "",
-                                    "trace": "",
-                                    "evaluation": evaluation.to_dict(),
-                                    "metadata": {},
-                                    "attempts": 0,
-                                }
-                            )
-                            step_results.append(
-                                StepResult(
-                                    step_id=step.id,
-                                    trace="",
-                                    output="",
-                                    evaluation=evaluation,
-                                    attempts=0,
-                                    metadata={},
-                                )
-                            )
-                            if captured_exception is None:
-                                captured_exception = outcome
-                            continue
+                        if captured_exception is None:
+                            captured_exception = outcome
+                        continue
 
-                        _store_outcome(step, outcome)
-                    if captured_exception is not None:
-                        raise captured_exception
+                    _store_outcome(step, outcome)
+                if captured_exception is not None:
+                    raise captured_exception
 
         organized_results = self._teacher.collect_results(step_summaries)
         final_answer = await self._student.asynthesize_final_answer(task, organized_results)
@@ -831,6 +759,7 @@ class Orchestrator:
         context = ExecutionContext.get()
         try:
             trajectory = self._build_session_trajectory(task, plan, result)
+            context.metadata["session_trajectory"] = self._serialize_session_trajectory(trajectory)
         except Exception as exc:  # pragma: no cover - defensive guard
             logger.exception("Failed to build session trajectory: %s", exc)
             return
@@ -841,8 +770,8 @@ class Orchestrator:
             return
         context.set_session_reward(
             evaluation.reward,
-            student_learning=evaluation.student_learning,
-            teacher_learning=evaluation.teacher_learning,
+            stats=evaluation.statistics,
+            audit=evaluation.audit,
         )
 
     def _build_session_trajectory(self, task: str, plan: Plan, result: Result) -> SessionTrajectory:
@@ -891,6 +820,34 @@ class Orchestrator:
             focus_prompt=None,
         )
 
+    @staticmethod
+    def _serialize_session_trajectory(trajectory: SessionTrajectory) -> Dict[str, Any]:
+        serialized_steps: List[Dict[str, Any]] = []
+        for record in trajectory.steps:
+            serialized_steps.append(
+                {
+                    "step": record.step.model_dump() if hasattr(record.step, "model_dump") else record.step,
+                    "trace": record.trace,
+                    "output": record.output,
+                    "attempts": record.attempts,
+                    "guidance": list(record.guidance) if record.guidance else None,
+                    "status": record.status,
+                    "validation": record.validation,
+                    "prior_results": record.prior_results,
+                    "metadata": record.metadata,
+                }
+            )
+        return {
+            "task": trajectory.task,
+            "final_answer": trajectory.final_answer,
+            "plan": trajectory.plan,
+            "steps": serialized_steps,
+            "execution_mode": trajectory.execution_mode,
+            "teacher_intervened": trajectory.teacher_intervened,
+            "session_metadata": trajectory.session_metadata,
+            "focus_prompt": trajectory.focus_prompt,
+        }
+
     def _determine_levels(self, plan: Plan) -> List[List[int]]:
         graph = DependencyGraph(plan)
         return graph.topological_levels()
@@ -1000,7 +957,6 @@ class Orchestrator:
     ) -> Dict[str, Any]:
         entry: Dict[str, Any] = {
             "output_text": output_text,
-            "status": structured_output.get("status"),
             "artifacts": self._ensure_jsonable((structured_output.get("result") or {}).get("artifacts") or {}),
             "deliverable": self._ensure_jsonable((structured_output.get("result") or {}).get("deliverable")),
         }
@@ -1010,6 +966,11 @@ class Orchestrator:
         entry["structured_output"] = self._ensure_jsonable(structured_output)
         return entry
 
+    def _prepare_cached_validation(self, payload: Dict[str, Any]) -> Dict[str, Any]:
+        cached = dict(payload)
+        cached.pop("cached", None)
+        return self._ensure_jsonable(cached)
+
     def _ensure_jsonable(self, value: Any, depth: int = 0) -> Any:
         if depth > 6:
             return str(value)
