@@ -3,41 +3,45 @@
 from __future__ import annotations
 
 import json
-import os
 from dataclasses import dataclass, field
-from typing import Any
-from typing import Dict
-from typing import List
-from typing import Sequence
+from typing import Any, Sequence
 
 try:
-    import litellm
+    import litellm  # type: ignore[import-untyped]
     _LITELLM_ERROR = None
     litellm.drop_params = True
 except ModuleNotFoundError as exc:
-    litellm = None
+    litellm = None  # type: ignore[assignment]
     _LITELLM_ERROR = exc
 
 from atlas.config.models import LLMParameters
+from atlas.utils.env import is_offline_mode
 
 
 @dataclass
 class LLMResponse:
     content: str
     raw: Any
-    reasoning: Dict[str, Any] = field(default_factory=dict)
+    reasoning: dict[str, Any] = field(default_factory=dict)
 
 
 class LLMClient:
     def __init__(self, parameters: LLMParameters) -> None:
         self._params = parameters
+        self._mock_mode = is_offline_mode()
+
+    @property
+    def model(self) -> str:
+        return self._params.model
 
     async def acomplete(
         self,
-        messages: Sequence[Dict[str, Any]],
-        response_format: Dict[str, Any] | None = None,
-        overrides: Dict[str, Any] | None = None,
+        messages: Sequence[dict[str, Any]],
+        response_format: dict[str, Any] | None = None,
+        overrides: dict[str, Any] | None = None,
     ) -> LLMResponse:
+        if self._mock_mode:
+            return self._mock_response(messages, response_format)
         self._ensure_client()
         kwargs = self._prepare_kwargs(messages, response_format, overrides)
         result = await litellm.acompletion(**kwargs)
@@ -46,10 +50,12 @@ class LLMClient:
 
     def complete(
         self,
-        messages: Sequence[Dict[str, Any]],
-        response_format: Dict[str, Any] | None = None,
-        overrides: Dict[str, Any] | None = None,
+        messages: Sequence[dict[str, Any]],
+        response_format: dict[str, Any] | None = None,
+        overrides: dict[str, Any] | None = None,
     ) -> LLMResponse:
+        if self._mock_mode:
+            return self._mock_response(messages, response_format)
         self._ensure_client()
         kwargs = self._prepare_kwargs(messages, response_format, overrides)
         result = litellm.completion(**kwargs)
@@ -58,16 +64,18 @@ class LLMClient:
 
     def _prepare_kwargs(
         self,
-        messages: Sequence[Dict[str, Any]],
-        response_format: Dict[str, Any] | None,
-        overrides: Dict[str, Any] | None,
-    ) -> Dict[str, Any]:
+        messages: Sequence[dict[str, Any]],
+        response_format: dict[str, Any] | None,
+        overrides: dict[str, Any] | None,
+    ) -> dict[str, Any]:
+        import os
+        
         params = self._params
         api_key = os.getenv(params.api_key_env)
         if not api_key:
             raise RuntimeError(f"Environment variable '{params.api_key_env}' is not set")
         overrides = overrides or {}
-        kwargs: Dict[str, Any] = {"model": params.model, "messages": list(messages), "api_key": api_key}
+        kwargs: dict[str, Any] = {"model": params.model, "messages": list(messages), "api_key": api_key}
         if params.api_base:
             kwargs["api_base"] = params.api_base
         if params.organization:
@@ -112,14 +120,14 @@ class LLMClient:
             raise RuntimeError("litellm is required for LLMClient operations") from _LITELLM_ERROR
 
 
-    def _extract_content(self, response: Any) -> tuple[str, Dict[str, Any]]:
+    def _extract_content(self, response: Any) -> tuple[str, dict[str, Any]]:
         try:
             choice = response["choices"][0]
             message = choice["message"]
             content = message.get("content")
             if content is None and "tool_calls" in message:
                 return json.dumps(message["tool_calls"]), {}
-            reasoning_payload: Dict[str, Any] = {}
+            reasoning_payload: dict[str, Any] = {}
             for key in ("reasoning_content", "thinking", "thinking_blocks"):
                 value = message.get(key)
                 if value:
@@ -146,11 +154,11 @@ class LLMClient:
         except (KeyError, IndexError, TypeError) as exc:
             raise RuntimeError(f"Unexpected response format from LLM client. Response: {response}") from exc
 
-    def _record_reasoning(self, payload: Dict[str, Any]) -> None:
+    def _record_reasoning(self, payload: dict[str, Any]) -> None:
         if not payload:
             return
         try:
-            from atlas.runtime.orchestration.execution_context import ExecutionContext  # noqa: WPS433
+            from atlas.runtime.orchestration.execution_context import ExecutionContext
             context = ExecutionContext.get()
         except Exception:  # pragma: no cover - context not initialised
             return
@@ -159,3 +167,35 @@ class LLMClient:
             return
         queue = context.metadata.setdefault("_llm_reasoning_queue", [])
         queue.append({"origin": origin, "payload": payload})
+
+    def _mock_response(
+        self,
+        messages: Sequence[dict[str, Any]],
+        response_format: dict[str, Any] | None,
+    ) -> LLMResponse:
+        user_content = ""
+        if messages:
+            last_message = messages[-1]
+            if isinstance(last_message, dict):
+                user_content = str(last_message.get("content", ""))
+        if response_format and response_format.get("type") == "json_object":
+            if "plan" in user_content:
+                payload = {
+                    "steps": [
+                        {
+                            "id": 1,
+                            "description": "Mock step generated by ATLAS_OFFLINE_MODE.",
+                            "depends_on": [],
+                            "tool": None,
+                            "tool_params": None,
+                        }
+                    ]
+                }
+            elif "guidance" in user_content:
+                payload = {"valid": True, "guidance": "No additional guidance required."}
+            else:
+                payload = {"notes": "Mock JSON response from ATLAS_OFFLINE_MODE."}
+            content = json.dumps(payload)
+        else:
+            content = "Mock response from ATLAS_OFFLINE_MODE."
+        return LLMResponse(content=content, reasoning={}, raw={"mock": True})
